from __future__ import annotations

from datetime import datetime
from typing import Any

from pydantic import BaseModel, Field


class RuntimeIdentity(BaseModel):
    """Runtime identity and declared capabilities."""

    runtime_id: str
    runtime_name: str
    runtime_version: str
    environment: str
    capabilities: list[str]


class RuntimeState(BaseModel):
    """Runtime-local state view."""

    current_digest: str | None = None
    realized_digests: list[str] = Field(default_factory=list)
    updated_at: datetime


class RealizationRequest(BaseModel):
    """Public realization request model for the test runtime."""

    candidate_digest: str
    payload: dict[str, Any] = Field(default_factory=dict)


class RealizationReceipt(BaseModel):
    """Public realization receipt returned by the test runtime."""

    digest: str
    status: str
    message: str = ""
    realized_at: datetime


from __future__ import annotations

from typing import Any, Mapping

import requests

DEFAULT_TIMEOUT = 10.0


class KeyholeClient:
    """Python client for interacting with a Keyhole-compatible runtime."""

    def __init__(
        self,
        base_url: str,
        *,
        timeout: float = DEFAULT_TIMEOUT,
        session: requests.Session | None = None,
    ) -> None:
        self.base_url = base_url.rstrip("/")
        self.timeout = timeout
        self.session = session or requests.Session()

    def _request(self, method: str, path: str, **kwargs: Any) -> dict[str, Any]:
        response = self.session.request(
            method,
            f"{self.base_url}{path}",
            timeout=self.timeout,
            **kwargs,
        )
        response.raise_for_status()
        return response.json()

    def health(self) -> dict[str, Any]:
        """Return runtime health information."""
        return self._request("GET", "/healthz")

    def identity(self) -> dict[str, Any]:
        """Return runtime identity and declared capabilities."""
        return self._request("GET", "/identity")

    def state(self) -> dict[str, Any]:
        """Return the current runtime-local state view."""
        return self._request("GET", "/state")

    def realize(
        self,
        candidate_digest: str,
        payload: Mapping[str, Any] | None = None,
    ) -> dict[str, Any]:
        """Submit a bounded realization request using the current public contract."""
        body = {
            "candidate_digest": candidate_digest,
            "payload": dict(payload or {}),
        }
        return self._request("POST", "/realize", json=body)

    def realize_request(self, request_body: Mapping[str, Any]) -> dict[str, Any]:
        """Submit a prebuilt realization request body."""
        return self._request("POST", "/realize", json=dict(request_body))

    def close(self) -> None:
        """Close the underlying HTTP session."""
        self.session.close()


# Backward-compatible alias for the current class name on main.
RuntimeBridgeClient = KeyholeClient


"""Keyhole SDK package.

Public Python client for interacting with Keyhole-compatible runtimes.
"""

from .client import KeyholeClient
from . import models

__all__ = [
    "KeyholeClient",
    "models",
]

__version__ = "0.1.0"